import { Map, List } from 'immutable';
import { sortBy, filterData, getSummary, prepareViewerData, calculateTimings } from '../../utils.js';
import { RESET, SET_TABLE_HEADER_WIDTH, SELECT_REQUEST, UPDATE_SCROLL_TO_INDEX, UPDATE_ERROR_MESSAGE, FETCH_FILE, UPDATE_SORT, UPDATE_TYPE_FILTER, UPDATE_STATUS_FILTER, UPDATE_SEARCH, UPDATE_DATA } from './types.js';
import { EMPTY_NETWORK_HAR, DEFAULT_STATUS_FILTER } from '../../constants.js';

var initialState = new Map({
  rawData: EMPTY_NETWORK_HAR,
  data: new List(),
  actualData: new List(),
  totalNetworkTime: null,
  dataSummary: new Map(),
  sort: {
    key: 'startedDateTime',
    isAsc: true
  },
  search: {
    name: 'URL',
    value: ''
  },
  statusFilter: DEFAULT_STATUS_FILTER,
  typeFilter: {
    name: null,
    value: null
  },
  error: null,
  loading: false,
  scrollToIndex: null,
  selectedReqIndex: null,
  showReqDetail: false,
  reqDetail: null,
  tableHeaderWidth: '100%',
  numberOfNewEntries: 0
});
var reducer = function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var _ref = arguments.length > 1 ? arguments[1] : undefined,
    type = _ref.type,
    payload = _ref.payload;
  switch (type) {
    case UPDATE_DATA:
      {
        return state.withMutations(function (newState) {
          var _prepareViewerData = prepareViewerData(payload.log.entries),
            data = _prepareViewerData.data,
            totalNetworkTime = _prepareViewerData.totalNetworkTime,
            totalRequests = _prepareViewerData.totalRequests,
            totalTransferredSize = _prepareViewerData.totalTransferredSize,
            totalUncompressedSize = _prepareViewerData.totalUncompressedSize,
            finishTime = _prepareViewerData.finishTime;
          var sort = state.get('sort');
          var sortedData = new List(sortBy(data, sort.key, sort.isAsc));
          var filteredData = filterData({
            data: sortedData,
            statusFilter: state.get('statusFilter'),
            typeFilter: state.get('typeFilter'),
            search: state.get('search')
          });
          var prevData = state.get('data');
          var numberOfNewEntries = filteredData.size - prevData.size;
          newState.set('error', null).set('rawData', payload).set('data', filteredData).set('actualData', sortedData).set('numberOfNewEntries', numberOfNewEntries).set('totalNetworkTime', totalNetworkTime).set('dataSummary', new Map({
            totalRequests: totalRequests,
            totalTransferredSize: totalTransferredSize,
            totalUncompressedSize: totalUncompressedSize,
            finishTime: finishTime,
            timings: calculateTimings(payload.log.pages),
            finish: finishTime
          }));
        });
      }
    case UPDATE_SEARCH:
      {
        return state.withMutations(function (newState) {
          var data = filterData({
            data: state.get('actualData'),
            statusFilter: state.get('statusFilter'),
            typeFilter: state.get('typeFilter'),
            search: payload
          });
          var summary = getSummary(data);
          newState.set('search', payload).set('data', data).setIn(['dataSummary', 'totalRequests'], summary.totalRequests).setIn(['dataSummary', 'totalTransferredSize'], summary.totalTransferredSize).setIn(['dataSummary', 'totalUncompressedSize'], summary.totalUncompressedSize);
        });
      }
    case UPDATE_STATUS_FILTER:
      {
        return state.withMutations(function (newState) {
          var data = filterData({
            data: state.get('actualData'),
            statusFilter: payload,
            typeFilter: state.get('typeFilter'),
            search: state.get('search')
          });
          var summary = getSummary(data);
          newState.set('statusFilter', payload).set('data', data).setIn(['dataSummary', 'totalRequests'], summary.totalRequests).setIn(['dataSummary', 'totalTransferredSize'], summary.totalTransferredSize).setIn(['dataSummary', 'totalUncompressedSize'], summary.totalUncompressedSize);
        });
      }
    case UPDATE_TYPE_FILTER:
      {
        return state.withMutations(function (newState) {
          var data = filterData({
            data: state.get('actualData'),
            statusFilter: state.get('statusFilter'),
            typeFilter: payload,
            search: state.get('search')
          });
          var summary = getSummary(data);
          newState.set('typeFilter', payload).set('data', data).setIn(['dataSummary', 'totalRequests'], summary.totalRequests).setIn(['dataSummary', 'totalTransferredSize'], summary.totalTransferredSize).setIn(['dataSummary', 'totalUncompressedSize'], summary.totalUncompressedSize);
        });
      }
    case UPDATE_SORT:
      {
        return state.withMutations(function (newState) {
          newState.set('sort', payload).set('data', sortBy(state.get('data'), payload.key, payload.isAsc));
        });
      }
    case FETCH_FILE.REQUEST:
      {
        return state.withMutations(function (newState) {
          newState.set('error', null).set('loading', true);
        });
      }
    case FETCH_FILE.SUCCESS:
      {
        return state.withMutations(function (newState) {
          newState.set('error', null).set('loading', false);
        });
      }
    case FETCH_FILE.FAILURE:
    case UPDATE_ERROR_MESSAGE:
      {
        return state.withMutations(function (newState) {
          newState.set('error', payload).set('loading', false);
        });
      }
    case UPDATE_SCROLL_TO_INDEX:
      {
        return state.withMutations(function (newState) {
          newState.set('selectedReqIndex', payload ? payload.index : null).set('reqDetail', payload);
        });
      }
    case SELECT_REQUEST:
      {
        return state.withMutations(function (newState) {
          newState.set('selectedReqIndex', payload ? payload.index : null).set('reqDetail', payload).set('showReqDetail', !!payload);
        });
      }
    case SET_TABLE_HEADER_WIDTH:
      {
        return state.withMutations(function (newState) {
          newState.set('tableHeaderWidth', "".concat(payload, "px"));
        });
      }
    case RESET:
      {
        return initialState;
      }
    default:
      return state;
  }
};

export { reducer as default, initialState, reducer };
//# sourceMappingURL=reducer.js.map
